<div class="table-row">
    <div class="table-cell text-left title">NETWORK</div>
    <div class="table-cell" ng-show="!arguments.network_cumul && !arguments.network_sum">Rx/s</div>
    <div class="table-cell" ng-show="!arguments.network_cumul && !arguments.network_sum">Tx/s</div>

    <div class="table-cell" ng-show="!arguments.network_cumul && arguments.network_sum"></div>
    <div class="table-cell" ng-show="!arguments.network_cumul && arguments.network_sum">Rx+Tx/s</div>

    <div class="table-cell" ng-show="arguments.network_cumul && !arguments.network_sum">Rx</div>
    <div class="table-cell" ng-show="arguments.network_cumul && !arguments.network_sum">Tx</div>

    <div class="table-cell" ng-show="arguments.network_cumul && arguments.network_sum"></div>
    <div class="table-cell" ng-show="arguments.network_cumul && arguments.network_sum">Rx+Tx</div>
</div>
<div class="table-row" ng-repeat="network in statsNetwork.networks | orderBy: 'interfaceName'">
    <div class="table-cell text-left">{{ network.interfaceName | min_size }}</div>
    <div class="table-cell" ng-show="!arguments.network_cumul && !arguments.network_sum">{{ arguments.byte ? (network.rx / network.time_since_update | bytes) : (network.rx / network.time_since_update | bits) }}</div>
    <div class="table-cell" ng-show="!arguments.network_cumul && !arguments.network_sum">{{ arguments.byte ? (network.tx / network.time_since_update | bytes) : (network.tx / network.time_since_update | bits) }}</div>

    <div class="table-cell" ng-show="!arguments.network_cumul && arguments.network_sum"></div>
    <div class="table-cell" ng-show="!arguments.network_cumul && arguments.network_sum">{{ arguments.byte ? (network.cx / network.time_since_update | bytes) : (network.cx / network.time_since_update | bits) }}</div>

    <div class="table-cell" ng-show="arguments.network_cumul && !arguments.network_sum">{{ arguments.byte ? (network.cumulativeRx | bytes) : (network.cumulativeRx | bits) }}</div>
    <div class="table-cell" ng-show="arguments.network_cumul && !arguments.network_sum">{{ arguments.byte ? (network.cumulativeTx | bytes) : (network.cumulativeTx | bits) }}</div>

    <div class="table-cell" ng-show="arguments.network_cumul && arguments.network_sum"></div>
    <div class="table-cell" ng-show="arguments.network_cumul && arguments.network_sum">{{ arguments.byte ? (network.cumulativeCx | bytes) : (network.cumulativeCx | bits) }}</div>
</div>
