<div class="table">
    <div class="table-row">
        <div class="table-cell text-left">active:</div>
        <div class="table-cell">{{ statsMem.active | bytes }}</div>
    </div>
    <div class="table-row">
        <div class="table-cell text-left">inactive:</div>
        <div class="table-cell">{{ statsMem.inactive | bytes }}</div>
    </div>
    <div class="table-row" ng-show="statsMem.buffers != undefined">
        <div class="table-cell text-left">buffers:</div>
        <div class="table-cell">{{ statsMem.buffers | bytes }}</div>
    </div>
    <div class="table-row" ng-show="statsMem.cached != undefined">
        <div class="table-cell text-left">cached:</div>
        <div class="table-cell">{{ statsMem.cached | bytes }}</div>
    </div>
</div>
