<div class="cpu-name">
    {{ statsQuicklook.cpu_name }}
</div>
<div class="table">
    <div class="table-row" ng-show="!arguments.percpu">
        <div class="table-cell text-left">CPU</div>
        <div class="table-cell">
            <div class="progress">
                <div class="progress-bar progress-bar-{{ statsQuicklook.getDecoration('cpu') }}" role="progressbar" aria-valuenow="{{ statsQuicklook.cpu }}" aria-valuemin="0" aria-valuemax="100" style="width: {{ statsQuicklook.cpu }}%;">
                    &nbsp;
                </div>
            </div>
        </div>
        <div class="table-cell">
            {{ statsQuicklook.cpu }}%
        </div>
    </div>
    <div class="table-row" ng-show="arguments.percpu" ng-repeat="percpu in statsQuicklook.percpus">
        <div class="table-cell text-left">CPU{{ percpu.number }}</div>
        <div class="table-cell">
            <div class="progress">
                <div class="progress-bar progress-bar-{{ statsQuicklook.getDecoration('cpu') }}" role="progressbar" aria-valuenow="{{ percpu.total }}" aria-valuemin="0" aria-valuemax="100" style="width: {{ percpu.total }}%;">
                    &nbsp;
                </div>
            </div>
        </div>
        <div class="table-cell">
            {{ percpu.total }}%
        </div>
    </div>
    <div class="table-row">
        <div class="table-cell text-left">MEM</div>
        <div class="table-cell">
            <div class="progress">
                <div class="progress-bar progress-bar-{{ statsQuicklook.getDecoration('mem') }}" role="progressbar" aria-valuenow="{{ statsQuicklook.mem }}" aria-valuemin="0" aria-valuemax="100" style="width: {{ statsQuicklook.mem }}%;">
                    &nbsp;
                </div>
            </div>
        </div>
        <div class="table-cell">
            {{ statsQuicklook.mem }}%
        </div>
    </div>
    <div class="table-row">
        <div class="table-cell text-left">SWAP</div>
        <div class="table-cell">
            <div class="progress">
                <div class="progress-bar progress-bar-{{ statsQuicklook.getDecoration('swap') }}" role="progressbar" aria-valuenow="{{ statsQuicklook.swap }}" aria-valuemin="0" aria-valuemax="100" style="width: {{ statsQuicklook.swap }}%;">
                    &nbsp;
                </div>
            </div>
        </div>
        <div class="table-cell">
            {{ statsQuicklook.swap }}%
        </div>
    </div>
</div>
