<div class="table-row">
    <div class="table-cell text-left title">DISK I/O</div>
    <div class="table-cell" ng-show="!arguments.diskio_iops">R/s</div>
    <div class="table-cell" ng-show="!arguments.diskio_iops">W/s</div>

    <div class="table-cell" ng-show="arguments.diskio_iops">IOR/s</div>
    <div class="table-cell" ng-show="arguments.diskio_iops">IOW/s</div>
</div>
<div class="table-row" ng-repeat="disk in statsDiskio.disks">
    <div class="table-cell text-left">{{(disk.alias ? disk.alias : disk.name) | min_size}}</div>
    <div class="table-cell" ng-show="!arguments.diskio_iops">{{disk.bitrate.txps }}</div>
    <div class="table-cell" ng-show="!arguments.diskio_iops">{{disk.bitrate.rxps }}</div>

    <div class="table-cell" ng-show="arguments.diskio_iops">{{disk.count.txps }}</div>
    <div class="table-cell" ng-show="arguments.diskio_iops">{{disk.count.rxps }}</div>
</div>
