<div class="row">
    <div class="col-sm-12 col-lg-24">{{help.version}} {{help.psutil_version}}</div>
</div>
<div class="row">&nbsp;</div>
<div class="row">
    <div class="col-sm-12 col-lg-24">{{help.configuration_file}}</div>
</div>
<div class="row">&nbsp;</div>
<div class="row">
    <div class="col-sm-12 col-lg-6">{{help.sort_auto}}</div>
    <div class="col-sm-12 col-lg-6">{{help.sort_network}}</div>
</div>
<div class="row">
    <div class="col-sm-12 col-lg-6">{{help.sort_cpu}}</div>
    <div class="col-sm-12 col-lg-6">{{help.show_hide_alert}}</div>
</div>
<div class="row">
    <div class="col-sm-12 col-lg-6">{{help.sort_mem}}</div>
    <div class="col-sm-12 col-lg-6">{{help.percpu}}</div>
</div>
<div class="row">
    <div class="col-sm-12 col-lg-6">{{help.sort_user}}</div>
    <div class="col-sm-12 col-lg-6">{{help.show_hide_ip}}</div>
</div>
<div class="row">
    <div class="col-sm-12 col-lg-6">{{help.sort_proc}}</div>
    <div class="col-sm-12 col-lg-6">{{help.enable_disable_docker}}</div>
</div>
<div class="row">
    <div class="col-sm-12 col-lg-6">{{help.sort_io}}</div>
    <div class="col-sm-12 col-lg-6">{{help.view_network_io_combination}}</div>
</div>
<div class="row">
    <div class="col-sm-12 col-lg-6">{{help.sort_cpu_times}}</div>
    <div class="col-sm-12 col-lg-6">{{help.view_cumulative_network}}</div>
</div>
<div class="row">
    <div class="col-sm-12 col-lg-6">{{help.show_hide_diskio}}</div>
    <div class="col-sm-12 col-lg-6">{{help.show_hide_filesytem_freespace}}</div>
</div>
<div class="row">
    <div class="col-sm-12 col-lg-6">{{help.show_hide_filesystem}}</div>
    <div class="col-sm-12 col-lg-6">{{help.show_hide_help}}</div>
</div>
<div class="row">
    <div class="col-sm-12 col-lg-6">{{help.show_hide_network}}</div>
    <div class="col-sm-12 col-lg-6">{{help.diskio_iops}}</div>
</div>
<div class="row">
    <div class="col-sm-12 col-lg-6">{{help.show_hide_sensors}}</div>
    <div class="col-sm-12 col-lg-6">{{help.show_hide_top_menu}}</div>
</div>
<div class="row">
    <div class="col-sm-12 col-lg-6">{{help.show_hide_left_sidebar}}</div>
    <div class="col-sm-12 col-lg-6">{{help.show_hide_amp}}</div>
</div>
<div class="row">
    <div class="col-sm-12 col-lg-6">{{help.enable_disable_process_stats}}</div>
    <div class="col-sm-12 col-lg-6">{{help.show_hide_irq}}</div>
</div>
<div class="row">
    <div class="col-sm-12 col-lg-6">{{help.enable_disable_gpu}}</div>
    <div class="col-sm-12 col-lg-6">{{help.enable_disable_mean_gpu}}</div>
</div>
<div class="row">
    <div class="col-sm-12 col-lg-6">{{help.enable_disable_quick_look}}</div>
    <div class="col-sm-12 col-lg-6"></div>
</div>
<div class="row">
    <div class="col-sm-12 col-lg-6">{{help.enable_disable_short_processname}}</div>
    <div class="col-sm-12 col-lg-6"></div>
</div>
<div class="row">
    <div class="col-sm-12 col-lg-6">{{help.enable_disable_ports}}</div>
    <div class="col-sm-12 col-lg-6"></div>
</div>
