<span class="title" ng-show="!statsAlert.hasAlerts()">No warning or critical alert detected</span>
<span class="title" ng-show="statsAlert.hasAlerts()">Warning or critical alerts (lasts {{statsAlert.count()}} entries)</span>
