<div class="gpu-name title">
    {{ statsGpu.name }}
</div>
<div class="table">
    <div class="table-row" ng-if="arguments.meangpu || statsGpu.gpus.length === 1">
        <div class="table-cell text-left">proc:</div>
        <div class="table-cell" ng-class="statsGpu.getMeanDecoration('proc')" ng-if="statsGpu.mean.proc">{{ statsGpu.mean.proc | number : 0 }}%</div>
        <div class="table-cell" ng-if="!statsGpu.mean.proc">N/A</div>
    </div>
    <div class="table-row" ng-if="arguments.meangpu || statsGpu.gpus.length === 1">
        <div class="table-cell text-left">mem:</div>
        <div class="table-cell" ng-class="statsGpu.getMeanDecoration('mem')" ng-if="statsGpu.mean.mem">{{ statsGpu.mean.mem | number : 0 }}%</div>
        <div class="table-cell" ng-if="!statsGpu.mean.mem">N/A</div>
    </div>
    <div class="table-row" ng-if="!arguments.meangpu && statsGpu.gpus.length > 1" ng-repeat="gpu in statsGpu.gpus">
        <div class="table-cell text-left">
            {{ gpu.gpu_id }}:
            <span ng-class="statsGpu.getDecoration(gpu.gpu_id, 'proc')" ng-if="gpu.proc">{{ gpu.proc | number : 0 }}%</span>
            <span ng-if="!gpu.proc">N/A</span>
            mem:
            <span ng-class="statsGpu.getDecoration(gpu.gpu_id, 'mem')" ng-if="gpu.mem">{{ gpu.mem | number : 0 }}%</span>
            <span ng-if="!gpu.mem">N/A</span>
        </div>
    </div>
</div>
