<div class="table">
    <div class="table-row">
        <div class="table-cell text-left title">SWAP</div>
        <div class="table-cell">{{ statsMemSwap.percent }}%</div>
    </div>
    <div class="table-row">
        <div class="table-cell text-left">total:</div>
        <div class="table-cell">{{ statsMemSwap.total | bytes }}</div>
    </div>
    <div class="table-row">
        <div class="table-cell text-left">used:</div>
        <div class="table-cell" ng-class="statsMemSwap.getDecoration('used')">
            {{ statsMemSwap.used | bytes }}
        </div>
    </div>
    <div class="table-row">
        <div class="table-cell text-left">free:</div>
        <div class="table-cell">{{ statsMemSwap.free | bytes }}</div>
    </div>
</div>
