<span class="title">TASKS</span>
<span>{{ statsProcessCount.total }} ({{ statsProcessCount.thread }} thr),</span>
<span>{{ statsProcessCount.running }} run,</span>
<span>{{ statsProcessCount.sleeping }} slp,</span>
<span>{{ statsProcessCount.stopped }} oth</span>
<span> sorted {{ sorter.auto ? 'automatically' : '' }} by {{ sorter.getColumnLabel(sorter.column) }}, flat view</span>
