<div class="table">
    <div class="table-row">
        <div class="table-cell text-left title">PER CPU</div>
        <div class="table-cell" ng-repeat="percpu in statsPerCpu.cpus">{{ percpu.total }}%</div>
    </div>
    <div class="table-row">
        <div class="table-cell text-left">user:</div>
        <div class="table-cell" ng-repeat="percpu in statsPerCpu.cpus" ng-class="statsPerCpu.getUserAlert(percpu)">
            {{ percpu.user }}%
        </div>
    </div>
    <div class="table-row">
        <div class="table-cell text-left">system:</div>
        <div class="table-cell" ng-repeat="percpu in statsPerCpu.cpus" ng-class="statsPerCpu.getSystemAlert(percpu)">
            {{ percpu.system }}%
        </div>
    </div>
    <div class="table-row">
        <div class="table-cell text-left">idle:</div>
        <div class="table-cell" ng-repeat="percpu in statsPerCpu.cpus">{{ percpu.idle }}%</div>
    </div>
    <div class="table-row" ng-show="statsPerCpu.cpus[0].iowait">
        <div class="table-cell text-left">iowait:</div>
        <div class="table-cell" ng-repeat="percpu in statsPerCpu.cpus" ng-class="statsPerCpu.getSystemAlert(percpu)">
            {{ percpu.iowait }}%
        </div>
    </div>
    <div class="table-row" ng-show="statsPerCpu.cpus[0].steal">
        <div class="table-cell text-left">steal:</div>
        <div class="table-cell" ng-repeat="percpu in statsPerCpu.cpus" ng-class="statsPerCpu.getSystemAlert(percpu)">
            {{ percpu.steal }}%
        </div>
    </div>
</div>
