<div class="table">
    <div class="table-row">
        <div class="table-cell text-left title">LOAD</div>
        <div class="table-cell">{{ statsLoad.cpucore }}-core</div>
    </div>
    <div class="table-row">
        <div class="table-cell text-left">1 min:</div>
        <div class="table-cell">
            {{ statsLoad.min1 | number : 2}}
        </div>
    </div>
    <div class="table-row">
        <div class="table-cell text-left">5 min:</div>
        <div class="table-cell" ng-class="statsLoad.getDecoration('min5')">
            {{ statsLoad.min5 | number : 2}}
        </div>
    </div>
    <div class="table-row">
        <div class="table-cell text-left">15 min:</div>
        <div class="table-cell" ng-class="statsLoad.getDecoration('min15')">
            {{ statsLoad.min15 | number : 2}}
        </div>
    </div>
</div>
