body {
    background: black;
    color: #BBB;
    font-family: "Lucida Sans Typewriter", "Lucida Console", Monaco, "Bitstream Vera Sans Mono", monospace;
}

.table {
    display: table;
    width: 100%;
    max-width:100%;
}
.table-row-group {
    display: table-row-group
}
.table-row {
    display: table-row;
}
.table-cell {
    display: table-cell;
    text-align: right;
}

.top-plugin {
    margin-bottom: 20px;
}

.plugin {
    margin-bottom: 20px;
}
.plugin.table-row-group .table-row:last-child .table-cell {
    padding-bottom: 20px;
}

.underline {
    text-decoration: underline
}
.bold {
    font-weight: bold;
}
.sort {
    font-weight: bold;
    color: white;
}
.sortable {
    cursor: pointer;
}
.text-right {
    text-align: right;
}
.text-left {
    text-align: left;
}
.sidebar .table-cell:not(.text-left) {
    padding-left: 10px;
}

/* Theme */

.title {
    font-weight: bold;
    color: white;
}
.highlight {
    font-weight: bold;
    color: #5D4062;
}
.ok, .status, .process {
    color: #3E7B04;
    /*font-weight: bold;*/
}
.ok_log {
    background-color: #3E7B04;
    color: white;
    /*font-weight: bold;*/
}
.max {
    color: #3E7B04;
    font-weight: bold;
}
.careful {
    color: #295183;
    font-weight: bold;
}
.careful_log {
    background-color: #295183;
    color: white;
    font-weight: bold;
}
.warning, .nice {
    color: #5D4062;
    font-weight: bold;
}
.warning_log {
    background-color: #5D4062;
    color: white;
    font-weight: bold;
}
.critical {
    color: #A30000;
    font-weight: bold;
}
.critical_log {
    background-color: #A30000;
    color: white;
    font-weight: bold;
}

/* Plugins */
#processlist .table-cell {
    padding: 0px 5px 0px 5px;
    white-space: nowrap;
}
#containers .table-cell {
    padding: 0px 10px 0px 10px;
    white-space: nowrap;
}

#quicklook .progress {
    margin-bottom: 0px;
    min-width: 100px;
    background-color: #000;
    height: 12px;
    border-radius: 0px;
    text-align: right;
}
#quicklook .progress-bar-ok {
    background-color: #3E7B04;
}
#quicklook .progress-bar-careful {
    background-color: #295183;
}
#quicklook .progress-bar-warning {
    background-color: #5D4062;
}
#quicklook .progress-bar-critical {
    background-color: #A30000;
}
#quicklook .cpu-name {
    white-space: nowrap;
    overflow: hidden;
    width: 100%;
    text-overflow: ellipsis;
}
#amps .process-result {
    max-width: 300px;
    overflow: hidden;
    white-space: pre-wrap;
    padding-left: 10px;
    text-overflow: ellipsis;
}

#gpu .gpu-name {
    white-space: nowrap;
    overflow: hidden;
    width: 100%;
    text-overflow: ellipsis;
}

/* Loading page */

#loading-page .glances-logo {
    background: url('../images/glances.png') no-repeat center center;
    background-size: contain;
}

@media (max-width: 750px) {
    #loading-page .glances-logo {
        height: 400px;
    }
}
@media (min-width: 750px) {
    #loading-page .glances-logo {
        height: 500px;
    }
}


/*
Loading animation
source : https://github.com/lukehaas/css-loaders
*/
#loading-page .loader:before,
#loading-page .loader:after,
#loading-page .loader {
    border-radius: 50%;
    width: 1em;
    height: 1em;
    -webkit-animation-fill-mode: both;
    animation-fill-mode: both;
    -webkit-animation: loader 1.8s infinite ease-in-out;
    animation: loader 1.8s infinite ease-in-out;
}
#loading-page .loader {
    margin: auto;
    font-size: 10px;
    position: relative;
    text-indent: -9999em;
    -webkit-animation-delay: 0.16s;
    animation-delay: 0.16s;
}
#loading-page .loader:before {
    left: -3.5em;
}
#loading-page .loader:after {
    left: 3.5em;
    -webkit-animation-delay: 0.32s;
    animation-delay: 0.32s;
}
#loading-page .loader:before,
#loading-page .loader:after {
    content: '';
    position: absolute;
    top: 0;
}
@-webkit-keyframes loader {
    0%, 80%, 100% {
        box-shadow: 0 2.5em 0 -1.3em #56CA69;
    }
    40% {
        box-shadow: 0 2.5em 0 0 #56CA69;
    }
}
@keyframes loader {
    0%, 80%, 100% {
        box-shadow: 0 2.5em 0 -1.3em #56CA69;
    }
    40% {
        box-shadow: 0 2.5em 0 0 #56CA69;
    }
}
